@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  */
 
-#include "tftpsubs.h"
+#include "common/tftpsubs.h"
 
 #ifndef lint
 static const char *copyright UNUSED =
@@ -89,6 +89,7 @@ int	f;
 u_short port;
 int	trace;
 int	verbose;
+int	literal;
 int	connected;
 const struct modes *mode;
 #ifdef WITH_READLINE
@@ -102,6 +103,9 @@ const char *prompt = "tftp> ";
 sigjmp_buf	toplevel;
 void	intr(int);
 struct	servent *sp;
+int portrange = 0;
+unsigned int portrange_from = 0;
+unsigned int portrange_to   = 0;
 
 void	get (int, char **);
 void	help (int, char **);
@@ -116,6 +120,7 @@ void	settimeout (int, char **);
 void	settrace (int, char **);
 void	setverbose (int, char **);
 void	status (int, char **);
+void	setliteral (int, char **);
 
 static void command (void);
 
@@ -154,6 +159,9 @@ struct cmd cmdtab[] = {
   { "trace",
     "toggle packet tracing",
     settrace },
+  { "literal",
+    "toggle literal mode, ignore ':' in file name",
+    setliteral },
   { "status",
     "show current status",
     status },
@@ -187,7 +195,7 @@ const char *program;
 
 static inline void usage(int errcode)
 {
-  fprintf(stderr, "Usage: %s [-v][-m mode] [host [port]] [-c command]\n", program);
+  fprintf(stderr, "Usage: %s [-v][-l][-m mode] [host [port]] [-c command]\n", program);
   exit(errcode);
 }
 
@@ -220,6 +228,9 @@ main(int argc, char *argv[])
 	  /* Print version and configuration to stdout and exit */
 	  printf("%s\n", TFTP_CONFIG_STR);
 	  exit(0);
+	case 'l':
+	  literal = 1;
+	  break;
 	case 'm':
 	  if ( ++arg >= argc )
 	    usage(EX_USAGE);
@@ -241,6 +252,16 @@ main(int argc, char *argv[])
 	case 'c':
 	  iscmd = 1;
 	  break;
+    case 'R':
+      if ( ++arg >= argc )
+           usage(EX_USAGE);
+      if ( sscanf(argv[arg], "%u:%u", &portrange_from, &portrange_to) != 2 ||
+            portrange_from > portrange_to || portrange_to > 65535 ) {
+         fprintf(stderr, "Bad port range: %s\n", argv[arg]); 
+         exit(EX_USAGE);
+      }
+      portrange = 1;
+      break;
 	case 'h':
 	default:
 	  usage(*optx == 'h' ? 0 : EX_USAGE);
@@ -276,7 +297,7 @@ main(int argc, char *argv[])
   }
   bzero((char *)&s_in, sizeof (s_in));
   s_in.sin_family = AF_INET;
-  if (bind(f, (struct sockaddr *)&s_in, sizeof (s_in)) < 0) {
+  if (pick_port_bind(f, &s_in, portrange_from, portrange_to)) {
     perror("tftp: bind");
     exit(EX_OSERR);
   }
@@ -304,8 +325,6 @@ main(int argc, char *argv[])
     (*c->handler)(pargc, pargv);
     exit(0);
   }
-  if (sigsetjmp(toplevel,1) != 0)
-    (void)putchar('\n');
   
 #ifdef WITH_READLINE
 #ifdef HAVE_READLINE_HISTORY_H
@@ -313,6 +332,8 @@ main(int argc, char *argv[])
 #endif
 #endif
   
+  if (sigsetjmp(toplevel,1) != 0)
+    (void)putchar('\n');
   command();
   
   return 0;		/* Never reached */
@@ -336,8 +357,10 @@ getmoreargs(const char *partial, const char *mprompt)
   
   elen = strlen(eline);
 
-  if (line)
+  if (line) {
 	  free(line);
+	  line = NULL;
+  }
   line = xmalloc(len+elen+1);
   strcpy(line, partial);
   strcpy(line+len, eline);
@@ -487,7 +510,7 @@ put(int argc, char *argv[])
 		return;
 	}
 	targ = argv[argc - 1];
-	if (strchr(argv[argc - 1], ':')) {
+	if (!literal && strchr(argv[argc - 1], ':')) {
 		struct hostent *hp;
 
 		for (n = 1; n < argc - 1; n++)
@@ -576,14 +599,14 @@ get(int argc, char *argv[])
 	}
 	if (!connected) {
 		for (n = 1; n < argc ; n++)
-			if (strchr(argv[n], ':') == 0) {
+			if (literal || strchr(argv[n], ':') == 0) {
 				getusage(argv[0]);
 				return;
 			}
 	}
 	for (n = 1; n < argc ; n++) {
 		src = strchr(argv[n], ':');
-		if (src == NULL)
+		if (literal || src == NULL)
 			src = argv[n];
 		else {
 			struct hostent *hp;
@@ -684,6 +707,14 @@ settimeout(int argc, char *argv[])
 		maxtimeout = t;
 }
 
+void
+setliteral(int argc, char *argv[])
+{
+	(void)argc; (void)argv;	/* Quiet unused warning */
+	literal = !literal;
+	printf("Literal mode %s.\n", literal ? "on" : "off");
+}
+
 void
 status(int argc, char *argv[])
 {
@@ -692,8 +723,8 @@ status(int argc, char *argv[])
 		printf("Connected to %s.\n", hostname);
 	else
 		printf("Not connected.\n");
-	printf("Mode: %s Verbose: %s Tracing: %s\n", mode->m_mode,
-		verbose ? "on" : "off", trace ? "on" : "off");
+	printf("Mode: %s Verbose: %s Tracing: %s Literal: %s\n", mode->m_mode,
+		verbose ? "on" : "off", trace ? "on" : "off", literal ? "on" : "off");
 	printf("Rexmt-interval: %d seconds, Max-timeout: %d seconds\n",
 		rexmtval, maxtimeout);
 }
@@ -734,8 +765,10 @@ command(void)
 
 	for (;;) {
 #ifdef WITH_READLINE
-	        if ( line )
+	        if ( line ) {
 			free(line);
+			line = NULL;
+		}
 	        line = readline(prompt);
 		if ( !line )
 			exit(0); /* EOF */
