@@ -1,5 +1,3 @@
-/* tftp-hpa: $Id$ */
-
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
 /*
@@ -62,7 +60,7 @@ static const char *rcsid UNUSED =
 #include <limits.h>
 #include <syslog.h>
 
-#include "tftpsubs.h"
+#include "common/tftpsubs.h"
 #include "recvfrom.h"
 #include "remap.h"
 
@@ -96,13 +94,13 @@ char		buf[PKTSIZE];
 char		ackbuf[PKTSIZE];
 unsigned int	max_blksize = MAX_SEGSIZE;
 
-struct	sockaddr_in from;
-int	fromlen;
-off_t	tsize;
-int     tsize_ok;
+struct sockaddr_in from;
+socklen_t       fromlen;
+off_t	        tsize;
+int             tsize_ok;
 
-int	ndirs;
-const char **dirs;
+int	        ndirs;
+const char    **dirs;
 
 int		secure = 0;
 int		cancreate = 0;
@@ -273,38 +271,6 @@ static int recv_time(int s, void *rbuf, int len, unsigned int flags,
   }
 }
 
-static int
-pick_port_bind(int sockfd, struct sockaddr_in *myaddr)
-{
-  unsigned int port, firstport;
-
-  firstport = portrange
-    ? portrange_from + rand() % (portrange_to-portrange_from+1)
-    : 0;
-
-  port = firstport;
-
-  do {
-    myaddr->sin_port = htons(port);
-    
-    if (bind(sockfd, (struct sockaddr *)myaddr, sizeof *myaddr) < 0) {
-      /* Some versions of Linux return EINVAL instead of EADDRINUSE */
-      if ( !(portrange && (errno == EINVAL || errno == EADDRINUSE)) )
-	return -1;
-
-      /* Normally, we shouldn't have to loop, but some situations involving
-	 aborted transfers make it possible. */
-    } else {
-      return 0;
-    }
-
-    port++;
-    if ( port > portrange_to )
-      port = portrange_from;
-  } while ( port != firstport );
-
-  return -1;
-}
 
 int
 main(int argc, char **argv)
@@ -317,6 +283,7 @@ main(int argc, char **argv)
   int n;
   int fd = 0;
   int standalone = 0;		/* Standalone (listen) mode */
+  int nodaemon = 0;		/* Do not detach process */
   char *address = NULL;		/* Address to listen to */
   pid_t pid;
   mode_t my_umask = 0;
@@ -340,7 +307,7 @@ main(int argc, char **argv)
 
   srand(time(NULL) ^ getpid());
   
-  while ((c = getopt(argc, argv, "cspvVla:B:u:U:r:t:T:R:m:")) != -1)
+  while ((c = getopt(argc, argv, "cspvVlLa:B:u:U:r:t:T:R:m:")) != -1)
     switch (c) {
     case 'c':
       cancreate = 1;
@@ -354,6 +321,10 @@ main(int argc, char **argv)
     case 'l':
       standalone = 1;
       break;
+    case 'L':
+      standalone = 1;
+      nodaemon = 1;
+      break;
     case 'a':
       address = optarg;
       break;
@@ -531,33 +502,11 @@ main(int argc, char **argv)
     }
 
     /* Daemonize this process */
-    {
-      pid_t f = fork();
-      int nfd;
-      if ( f > 0 )
-	exit(0);
-      if ( f < 0 ) {
-	syslog(LOG_ERR, "cannot fork: %m");
-	exit(EX_OSERR);
-      }
-      nfd = open("/dev/null", O_RDWR);
-      if ( nfd >= 3 ) {
-#ifdef HAVE_DUP2
-	dup2(nfd, 0);
-	dup2(nfd, 1);
-	dup2(nfd, 2);
-#else
-	close(0); dup(nfd);
-	close(1); dup(nfd);
-	close(2); dup(nfd);
-#endif
-	close(nfd);
-      } else if ( nfd < 0 ) {
-	close(0); close(1); close(2);
-      }
-#ifdef HAVE_SETSID
-      setsid();
-#endif
+    /* Note: when running in secure mode (-s), we must not chroot, since
+       we are already in the proper directory. */
+    if (!nodaemon && daemon(secure, 0) < 0) {
+      syslog(LOG_ERR, "cannot daemonize: %m");
+      exit(EX_OSERR);
     }
   } else {
     /* 0 is our socket descriptor */
@@ -587,7 +536,7 @@ main(int argc, char **argv)
     if ( caught_sighup ) {
       caught_sighup = 0;
       if ( standalone ) {
-#ifdef HAVE_REGEX
+#ifdef WITH_REGEX
 	if ( rewrite_file ) {
 	  freerules(rewrite_rules);
 	  rewrite_rules = read_remap_rules(rewrite_file);
@@ -750,7 +699,7 @@ main(int argc, char **argv)
   from.sin_family = AF_INET;
   
   /* Process the request... */
-  if (pick_port_bind(peer, &myaddr) < 0) {
+  if (pick_port_bind(peer, &myaddr, portrange_from, portrange_to) < 0) {
     syslog(LOG_ERR, "bind: %m");
     exit(EX_IOERR);
   }
@@ -1078,8 +1027,7 @@ rewrite_macros(char macro, char *output)
     
   case 'x':
     if ( output )
-      sprintf(output, "%08X",
-	      ntohl(from.sin_addr.s_addr));
+      sprintf(output, "%08lX", (unsigned long)ntohl(from.sin_addr.s_addr));
     return 8;
 
   default:
